@@ -4,8 +4,8 @@ var classtvg_1_1Picture =
     [ "load", "classtvg_1_1Picture.html#a95979a7c4ce51445c7ef9d6461c34767", null ],
     [ "load", "classtvg_1_1Picture.html#aaf06be7d0b572c5ce35315cbddd7d318", null ],
     [ "load", "classtvg_1_1Picture.html#a1fdf75092cebaa7cde48d6a7d4946368", null ],
-    [ "mesh", "classtvg_1_1Picture.html#ae4d1479dbae28f521da8e90f91a97caa", null ],
-    [ "mesh", "classtvg_1_1Picture.html#a761d19e78330e8ce73add01cbf1cd2e9", null ],
+    [ "paint", "classtvg_1_1Picture.html#aeaff657c918ea2a3d984478a5e981ea2", null ],
     [ "size", "classtvg_1_1Picture.html#af80639ef3460a5cf06278fdc03c13aad", null ],
-    [ "size", "classtvg_1_1Picture.html#a065e47b46c791ee4d93ed3252d581a40", null ]
+    [ "size", "classtvg_1_1Picture.html#a065e47b46c791ee4d93ed3252d581a40", null ],
+    [ "type", "classtvg_1_1Picture.html#a0998d9f221df277cafde4934e3828aa2", null ]
 ];
\ No newline at end of file
