@@ -1,9 +1,7 @@
 var hierarchy =
 [
     [ "Accessor", "classtvg_1_1Accessor.html", null ],
-    [ "Animation", "classtvg_1_1Animation.html", [
-      [ "LottieAnimation", "classtvg_1_1LottieAnimation.html", null ]
-    ] ],
+    [ "Animation", "classtvg_1_1Animation.html", null ],
     [ "Canvas", "classtvg_1_1Canvas.html", [
       [ "GlCanvas", "classtvg_1_1GlCanvas.html", null ],
       [ "SwCanvas", "classtvg_1_1SwCanvas.html", null ],
@@ -23,7 +21,5 @@ var hierarchy =
       [ "Text", "classtvg_1_1Text.html", null ]
     ] ],
     [ "Point", "structtvg_1_1Point.html", null ],
-    [ "Polygon", "structtvg_1_1Polygon.html", null ],
-    [ "Saver", "classtvg_1_1Saver.html", null ],
-    [ "Vertex", "structtvg_1_1Vertex.html", null ]
+    [ "Saver", "classtvg_1_1Saver.html", null ]
 ];
\ No newline at end of file
