@@ -52,6 +52,12 @@ TVG_API Tvg_Result tvg_engine_term(Tvg_Engine engine_method)
 }
 
 
+TVG_API Tvg_Result tvg_engine_version(uint32_t* major, uint32_t* minor, uint32_t* micro, const char** version)
+{
+    if (version) *version = Initializer::version(major, minor, micro);
+    return TVG_RESULT_SUCCESS;
+}
+
 /************************************************************************/
 /* Canvas API                                                           */
 /************************************************************************/
@@ -111,13 +117,6 @@ TVG_API Tvg_Result tvg_canvas_update(Tvg_Canvas* canvas)
 }
 
 
-TVG_API Tvg_Result tvg_canvas_est_viewport(Tvg_Canvas* canvas, int32_t x, int32_t y, int32_t w, int32_t h)
-{
-    if (!canvas) return TVG_RESULT_INVALID_ARGUMENT;
-    return (Tvg_Result) reinterpret_cast<Canvas*>(canvas)->viewport(x, y, w, h);
-}
-
-
 TVG_API Tvg_Result tvg_canvas_update_paint(Tvg_Canvas* canvas, Tvg_Paint* paint)
 {
     if (!canvas || !paint) return TVG_RESULT_INVALID_ARGUMENT;
@@ -238,26 +237,31 @@ TVG_API Tvg_Result tvg_paint_get_composite_method(const Tvg_Paint* paint, const
 }
 
 
-TVG_API Tvg_Result tvg_paint_set_blend_method(const Tvg_Paint* paint, Tvg_Blend_Method method)
+TVG_API Tvg_Result tvg_paint_set_blend_method(Tvg_Paint* paint, Tvg_Blend_Method method)
 {
    if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
-   return (Tvg_Result) reinterpret_cast<const Paint*>(paint)->blend((BlendMethod)method);
+   return (Tvg_Result) reinterpret_cast<Paint*>(paint)->blend((BlendMethod)method);
 }
 
 
-TVG_API Tvg_Result tvg_paint_get_blend_method(const Tvg_Paint* paint, Tvg_Blend_Method* method)
+TVG_API Tvg_Result tvg_paint_get_type(const Tvg_Paint* paint, Tvg_Type* type)
 {
-    if (!paint || !method) return TVG_RESULT_INVALID_ARGUMENT;
-    *method = static_cast<Tvg_Blend_Method>(reinterpret_cast<const Paint*>(paint)->blend());
+    if (!paint || !type) return TVG_RESULT_INVALID_ARGUMENT;
+    *type = static_cast<Tvg_Type>(reinterpret_cast<const Paint*>(paint)->type());
     return TVG_RESULT_SUCCESS;
 }
 
 
-TVG_API Tvg_Result tvg_paint_get_identifier(const Tvg_Paint* paint, Tvg_Identifier* identifier)
+TVG_API Tvg_Result tvg_paint_set_clip(Tvg_Paint* paint, Tvg_Paint* clipper)
 {
-    if (!paint || !identifier) return TVG_RESULT_INVALID_ARGUMENT;
-    *identifier = static_cast<Tvg_Identifier>(reinterpret_cast<const Paint*>(paint)->identifier());
-    return TVG_RESULT_SUCCESS;
+   if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
+   return (Tvg_Result) reinterpret_cast<Paint*>(paint)->clip(unique_ptr<Paint>((Paint*)(clipper)));
+}
+
+
+TVG_DEPRECATED TVG_API Tvg_Result tvg_paint_get_identifier(const Tvg_Paint* paint, Tvg_Identifier* identifier)
+{
+    return tvg_paint_get_type(paint, (Tvg_Type*) identifier);
 }
 
 /************************************************************************/
@@ -312,7 +316,7 @@ TVG_API Tvg_Result tvg_shape_append_rect(Tvg_Paint* paint, float x, float y, flo
 }
 
 
-TVG_API Tvg_Result tvg_shape_append_arc(Tvg_Paint* paint, float cx, float cy, float radius, float startAngle, float sweep, uint8_t pie)
+TVG_DEPRECATED TVG_API Tvg_Result tvg_shape_append_arc(Tvg_Paint* paint, float cx, float cy, float radius, float startAngle, float sweep, uint8_t pie)
 {
     if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
     return (Tvg_Result) reinterpret_cast<Shape*>(paint)->appendArc(cx, cy, radius, startAngle, sweep, pie);
@@ -460,6 +464,13 @@ TVG_API Tvg_Result tvg_shape_get_stroke_miterlimit(const Tvg_Paint* paint, float
 }
 
 
+TVG_API Tvg_Result tvg_shape_set_stroke_trim(Tvg_Paint* paint, float begin, float end, bool simultaneous)
+{
+    if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
+    return (Tvg_Result) reinterpret_cast<Shape*>(paint)->strokeTrim(begin, end, simultaneous);
+}
+
+
 TVG_API Tvg_Result tvg_shape_set_fill_color(Tvg_Paint* paint, uint8_t r, uint8_t g, uint8_t b, uint8_t a)
 {
     if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
@@ -562,6 +573,13 @@ TVG_API Tvg_Result tvg_picture_get_size(const Tvg_Paint* paint, float* w, float*
 }
 
 
+TVG_API const Tvg_Paint* tvg_picture_get_paint(Tvg_Paint* paint, uint32_t id)
+{
+    if (!paint) return nullptr;
+    return (Tvg_Paint*) reinterpret_cast<Picture*>(paint)->paint(id);
+}
+
+
 /************************************************************************/
 /* Gradient API                                                         */
 /************************************************************************/
@@ -666,13 +684,19 @@ TVG_API Tvg_Result tvg_gradient_get_transform(const Tvg_Gradient* grad, Tvg_Matr
 }
 
 
-TVG_API Tvg_Result tvg_gradient_get_identifier(const Tvg_Gradient* grad, Tvg_Identifier* identifier)
+TVG_API Tvg_Result tvg_gradient_get_type(const Tvg_Gradient* grad, Tvg_Type* type)
 {
-    if (!grad || !identifier) return TVG_RESULT_INVALID_ARGUMENT;
-    *identifier = static_cast<Tvg_Identifier>(reinterpret_cast<const Fill*>(grad)->identifier());
+    if (!grad || !type) return TVG_RESULT_INVALID_ARGUMENT;
+    *type = static_cast<Tvg_Type>(reinterpret_cast<const Fill*>(grad)->type());
     return TVG_RESULT_SUCCESS;
 }
 
+
+TVG_DEPRECATED TVG_API Tvg_Result tvg_gradient_get_identifier(const Tvg_Gradient* grad, Tvg_Identifier* identifier)
+{
+    return tvg_gradient_get_type(grad, (Tvg_Type*) identifier);
+}
+
 /************************************************************************/
 /* Scene API                                                            */
 /************************************************************************/
@@ -703,6 +727,62 @@ TVG_API Tvg_Result tvg_scene_clear(Tvg_Paint* scene, bool free)
 }
 
 
+/************************************************************************/
+/* Text API                                                            */
+/************************************************************************/
+
+TVG_API Tvg_Paint* tvg_text_new()
+{
+    return (Tvg_Paint*)Text::gen().release();
+}
+
+
+TVG_API Tvg_Result tvg_text_set_font(Tvg_Paint* paint, const char* name, float size, const char* style)
+{
+    if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
+    return (Tvg_Result) reinterpret_cast<Text*>(paint)->font(name, size, style);
+}
+
+
+TVG_API Tvg_Result tvg_text_set_text(Tvg_Paint* paint, const char* text)
+{
+    if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
+    return (Tvg_Result) reinterpret_cast<Text*>(paint)->text(text);
+}
+
+
+TVG_API Tvg_Result tvg_text_set_fill_color(Tvg_Paint* paint, uint8_t r, uint8_t g, uint8_t b)
+{
+    if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
+    return (Tvg_Result) reinterpret_cast<Text*>(paint)->fill(r, g, b);
+}
+
+
+TVG_API Tvg_Result tvg_text_set_gradient(Tvg_Paint* paint, Tvg_Gradient* gradient)
+{
+    if (!paint) return TVG_RESULT_INVALID_ARGUMENT;
+    return (Tvg_Result) reinterpret_cast<Text*>(paint)->fill(unique_ptr<Fill>((Fill*)(gradient)));
+}
+
+
+TVG_API Tvg_Result tvg_font_load(const char* path)
+{
+    return (Tvg_Result) Text::load(path);
+}
+
+
+TVG_API Tvg_Result tvg_font_load_data(const char* name, const char* data, uint32_t size, const char *mimetype, bool copy)
+{
+    return (Tvg_Result) Text::load(name, data, size, mimetype ? mimetype : "", copy);
+}
+
+
+TVG_API Tvg_Result tvg_font_unload(const char* path)
+{
+    return (Tvg_Result) Text::unload(path);
+}
+
+
 /************************************************************************/
 /* Saver API                                                            */
 /************************************************************************/
@@ -805,6 +885,16 @@ TVG_API Tvg_Result tvg_animation_del(Tvg_Animation* animation)
 }
 
 
+/************************************************************************/
+/* Accessor API                                                         */
+/************************************************************************/
+
+TVG_API uint32_t tvg_accessor_generate_id(const char* name)
+{
+    return Accessor::id(name);
+}
+
+
 /************************************************************************/
 /* Lottie Animation API                                                 */
 /************************************************************************/
