@@ -136,45 +136,30 @@ static jerry_value_t _toComp(const jerry_call_info_t* info, const jerry_value_t
 }
 
 
-static jerry_value_t _value(float frameNo, LottieProperty* property)
+static jerry_value_t _point(const Point& v)
+{
+    auto obj = _point2d(v);
+    auto value = _point2d(v);
+    jerry_object_set_sz(obj, EXP_VALUE, value);
+    jerry_value_free(value);
+    return obj;
+}
+
+
+static jerry_value_t _buildValue(float frameNo, LottieProperty* property)
 {
     switch (property->type) {
-        case LottieProperty::Type::Point: {
-            auto value = jerry_object();
-            auto pos = (*static_cast<LottiePoint*>(property))(frameNo);
-            auto val1 = jerry_number(pos.x);
-            auto val2 = jerry_number(pos.y);
-            jerry_object_set_index(value, 0, val1);
-            jerry_object_set_index(value, 1, val2);
-            jerry_value_free(val1);
-            jerry_value_free(val2);
-            return value;
-        }
-        case LottieProperty::Type::Float: {
-            return jerry_number((*static_cast<LottieFloat*>(property))(frameNo));
-        }
-        case LottieProperty::Type::Opacity: {
-            return jerry_number((*static_cast<LottieOpacity*>(property))(frameNo));
-        }
-        case LottieProperty::Type::PathSet: {
-            auto value = jerry_object();
-            jerry_object_set_native_ptr(value, nullptr, property);
-            return value;
-        }
-        case LottieProperty::Type::Position: {
-            auto value = jerry_object();
-            auto pos = (*static_cast<LottiePosition*>(property))(frameNo);
-            auto val1 = jerry_number(pos.x);
-            auto val2 = jerry_number(pos.y);
-            jerry_object_set_index(value, 0, val1);
-            jerry_object_set_index(value, 1, val2);
-            jerry_value_free(val1);
-            jerry_value_free(val2);
-            return value;
-        }
-        default: {
-            TVGERR("LOTTIE", "Non supported type for value? = %d", (int) property->type);
+        case LottieProperty::Type::Float: return jerry_number((*static_cast<LottieFloat*>(property))(frameNo));
+        case LottieProperty::Type::Point: return _point((*static_cast<LottiePoint*>(property))(frameNo));
+        case LottieProperty::Type::Position: return _point((*static_cast<LottiePosition*>(property))(frameNo));
+        case LottieProperty::Type::PathSet:
+        {
+            auto obj = jerry_object();
+            jerry_object_set_native_ptr(obj, nullptr, property);
+            return obj;
         }
+        case LottieProperty::Type::Opacity: return jerry_number((*static_cast<LottieOpacity*>(property))(frameNo));
+        default: TVGERR("LOTTIE", "Non supported type for value? = %d", (int) property->type);
     }
     return jerry_undefined();
 }
@@ -187,23 +172,23 @@ static void _buildTransform(jerry_value_t context, float frameNo, LottieTransfor
     auto obj = jerry_object();
     jerry_object_set_sz(context, "transform", obj);
 
-    auto anchorPoint = _value(frameNo, &transform->anchor);
+    auto anchorPoint = _buildValue(frameNo, &transform->anchor);
     jerry_object_set_sz(obj, "anchorPoint", anchorPoint);
     jerry_value_free(anchorPoint);
 
-    auto position = _value(frameNo, &transform->position);
+    auto position = _buildValue(frameNo, &transform->position);
     jerry_object_set_sz(obj, "position", position);
     jerry_value_free(position);
 
-    auto scale = _value(frameNo, &transform->scale);
+    auto scale = _buildValue(frameNo, &transform->scale);
     jerry_object_set_sz(obj, "scale", scale);
     jerry_value_free(scale);
 
-    auto rotation = _value(frameNo, &transform->rotation);
+    auto rotation = _buildValue(frameNo, &transform->rotation);
     jerry_object_set_sz(obj, "rotation", rotation);
     jerry_value_free(rotation);
 
-    auto opacity = _value(frameNo, &transform->opacity);
+    auto opacity = _buildValue(frameNo, &transform->opacity);
     jerry_object_set_sz(obj, "opacity", opacity);
     jerry_value_free(opacity);
 
@@ -736,7 +721,7 @@ static jerry_value_t _property(const jerry_call_info_t* info, const jerry_value_
     auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
     auto property = data->obj->property(jerry_value_as_int32(args[0]));
     if (!property) return jerry_undefined();
-    return _value(data->frameNo, property);
+    return _buildValue(data->frameNo, property);
 }
 
 
@@ -764,7 +749,7 @@ static jerry_value_t _valueAtTime(const jerry_call_info_t* info, const jerry_val
     auto exp = static_cast<LottieExpression*>(jerry_object_get_native_ptr(info->function, nullptr));
     auto time = jerry_value_as_number(args[0]);
     auto frameNo = exp->comp->frameAtTime(time);
-    return _value(frameNo, exp->property);
+    return _buildValue(frameNo, exp->property);
 }
 
 
@@ -878,106 +863,61 @@ static jerry_value_t _wiggle(const jerry_call_info_t* info, const jerry_value_t
 }
 
 
-static bool _loopOutCommon(LottieExpression* exp, const jerry_value_t args[], const jerry_length_t argsCnt)
+static LottieProperty::Loop _loopCommon(const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    exp->loop.mode = LottieExpression::LoopMode::OutCycle;
+    auto mode = LottieProperty::Loop::InCycle;
 
     if (argsCnt > 0) {
         auto name = _name(args[0]);
-        if (!strcmp(name, EXP_CYCLE)) exp->loop.mode = LottieExpression::LoopMode::OutCycle;
-        else if (!strcmp(name, EXP_PINGPONG)) exp->loop.mode = LottieExpression::LoopMode::OutPingPong;
-        else if (!strcmp(name, EXP_OFFSET)) exp->loop.mode = LottieExpression::LoopMode::OutOffset;
-        else if (!strcmp(name, EXP_CONTINUE)) exp->loop.mode = LottieExpression::LoopMode::OutContinue;
+        if (!strcmp(name, EXP_CYCLE)) mode = LottieProperty::Loop::InCycle;
+        else if (!strcmp(name, EXP_PINGPONG)) mode = LottieProperty::Loop::InPingPong;
+        else if (!strcmp(name, EXP_OFFSET)) mode = LottieProperty::Loop::InOffset;
+        else if (!strcmp(name, EXP_CONTINUE)) mode = LottieProperty::Loop::InContinue;
         free(name);
     }
 
-    if (exp->loop.mode != LottieExpression::LoopMode::OutCycle && exp->loop.mode != LottieExpression::LoopMode::OutPingPong) {
-        TVGLOG("LOTTIE", "Not supported loopOut type = %d", exp->loop.mode);
-        return false;
+    if (mode != LottieProperty::Loop::InCycle && mode != LottieProperty::Loop::InPingPong) {
+        TVGLOG("LOTTIE", "Not supported loopIn type = %d", (int) mode);
     }
-
-    return true;
+    return mode;
 }
 
 
+#define LOOP_OUT_OFFSET 4
+
 static jerry_value_t _loopOut(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto exp = static_cast<LottieExpression*>(jerry_object_get_native_ptr(info->function, nullptr));
-
-    if (!_loopOutCommon(exp, args, argsCnt)) return jerry_undefined();
-
-    if (argsCnt > 1) exp->loop.key = jerry_value_as_int32(args[1]);
-
-    auto obj = jerry_object();
-    jerry_object_set_native_ptr(obj, nullptr, exp->property);
-    return obj;
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto mode = static_cast<LottieProperty::Loop>((int) _loopCommon(args, argsCnt) + LOOP_OUT_OFFSET);
+    auto key = (argsCnt > 1) ? jerry_value_as_int32(args[1]) : 0;
+    return _buildValue(data->exp->property->loop(data->frameNo, key, mode, data->exp->layer->outFrame), data->exp->property);
 }
 
 
 static jerry_value_t _loopOutDuration(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto exp = static_cast<LottieExpression*>(jerry_object_get_native_ptr(info->function, nullptr));
-
-    if (!_loopOutCommon(exp, args, argsCnt)) return jerry_undefined();
-
-    if (argsCnt > 1) {
-        exp->loop.in = exp->comp->frameAtTime((float)jerry_value_as_int32(args[1]));
-    }
-
-    auto obj = jerry_object();
-    jerry_object_set_native_ptr(obj, nullptr, exp->property);
-    return obj;
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto mode = static_cast<LottieProperty::Loop>((int) _loopCommon(args, argsCnt) + LOOP_OUT_OFFSET);
+    auto out = (argsCnt > 1) ? data->exp->comp->frameAtTime(jerry_value_as_number(args[1])) : FLT_MAX;
+    return _buildValue(data->exp->property->loop(data->frameNo, 0, mode, out), data->exp->property);
 }
 
 
-static bool _loopInCommon(LottieExpression* exp, const jerry_value_t args[], const jerry_length_t argsCnt)
-{
-    exp->loop.mode = LottieExpression::LoopMode::InCycle;
-
-    if (argsCnt > 0) {
-        auto name = _name(args[0]);
-        if (!strcmp(name, EXP_CYCLE)) exp->loop.mode = LottieExpression::LoopMode::InCycle;
-        else if (!strcmp(name, EXP_PINGPONG)) exp->loop.mode = LottieExpression::LoopMode::InPingPong;
-        else if (!strcmp(name, EXP_OFFSET)) exp->loop.mode = LottieExpression::LoopMode::InOffset;
-        else if (!strcmp(name, EXP_CONTINUE)) exp->loop.mode = LottieExpression::LoopMode::InContinue;
-        free(name);
-    }
-
-    if (exp->loop.mode != LottieExpression::LoopMode::InCycle && exp->loop.mode != LottieExpression::LoopMode::InPingPong) {
-        TVGLOG("LOTTIE", "Not supported loopIn type = %d", exp->loop.mode);
-        return false;
-    }
-
-    return true;
-}
-
 static jerry_value_t _loopIn(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto exp = static_cast<LottieExpression*>(jerry_object_get_native_ptr(info->function, nullptr));
-
-    if (!_loopInCommon(exp, args, argsCnt)) return jerry_undefined();
-
-    if (argsCnt > 1) exp->loop.key = jerry_value_as_int32(args[1]);
-
-    auto obj = jerry_object();
-    jerry_object_set_native_ptr(obj, nullptr, exp->property);
-    return obj;
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto mode = _loopCommon(args, argsCnt);
+    auto key = (argsCnt > 1) ? jerry_value_as_int32(args[1]) : 0;
+    return _buildValue(data->exp->property->loop(data->frameNo, key, mode, data->exp->layer->outFrame), data->exp->property);
 }
 
 
 static jerry_value_t _loopInDuration(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto exp = static_cast<LottieExpression*>(jerry_object_get_native_ptr(info->function, nullptr));
-
-    if (argsCnt > 1) {
-        exp->loop.in = exp->comp->frameAtTime((float)jerry_value_as_int32(args[1]));
-    }
-
-    if (!_loopInCommon(exp, args, argsCnt)) return jerry_undefined();
-
-    auto obj = jerry_object();
-    jerry_object_set_native_ptr(obj, nullptr, exp->property);
-    return obj;
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto mode = _loopCommon(args, argsCnt);
+    auto in = (argsCnt > 1) ? data->exp->comp->frameAtTime(jerry_value_as_number(args[1])) : FLT_MAX;
+    return _buildValue(data->exp->property->loop(data->frameNo, 0, mode, in), data->exp->property);
 }
 
 
@@ -987,7 +927,7 @@ static jerry_value_t _key(const jerry_call_info_t* info, const jerry_value_t arg
     auto key = jerry_value_as_int32(args[0]);
     auto frameNo = exp->property->frameNo(key);
     auto time = jerry_number(exp->comp->timeAtFrame(frameNo));
-    auto value = _value(frameNo, exp->property);
+    auto value = _buildValue(frameNo, exp->property);
 
     auto obj = jerry_object();
     jerry_object_set_sz(obj, EXP_TIME, time);
@@ -1074,7 +1014,7 @@ static void _buildPath(jerry_value_t context, LottieExpression* exp)
 
 static void _buildProperty(float frameNo, jerry_value_t context, LottieExpression* exp)
 {
-    auto value = _value(frameNo, exp->property);
+    auto value = _buildValue(frameNo, exp->property);
     jerry_object_set_sz(context, EXP_VALUE, value);
     jerry_value_free(value);
 
@@ -1102,7 +1042,7 @@ static void _buildProperty(float frameNo, jerry_value_t context, LottieExpressio
     jerry_value_free(speedAtTime);
 
     {
-        auto data =  _expcontent(exp, frameNo, exp->object, 2);
+        auto data =  _expcontent(exp, frameNo, exp->object, 6);
 
         auto wiggle = jerry_function_external(_wiggle);
         jerry_object_set_sz(context, "wiggle", wiggle);
@@ -1117,29 +1057,30 @@ static void _buildProperty(float frameNo, jerry_value_t context, LottieExpressio
         jerry_value_free(propertyGroup);
 
         //propertyIndex
+
+        auto loopIn = jerry_function_external(_loopIn);
+        jerry_object_set_sz(context, "loopIn", loopIn);
+        jerry_object_set_native_ptr(loopIn, &freeCb, data);
+        jerry_value_free(loopIn);
+
+        auto loopOut = jerry_function_external(_loopOut);
+        jerry_object_set_sz(context, "loopOut", loopOut);
+        jerry_object_set_native_ptr(loopOut, &freeCb, data);
+        jerry_value_free(loopOut);
+
+        auto loopInDuration = jerry_function_external(_loopInDuration);
+        jerry_object_set_sz(context, "loopInDuration", loopInDuration);
+        jerry_object_set_native_ptr(loopInDuration, &freeCb, data);
+        jerry_value_free(loopInDuration);
+
+        auto loopOutDuration = jerry_function_external(_loopOutDuration);
+        jerry_object_set_sz(context, "loopOutDuration", loopOutDuration);
+        jerry_object_set_native_ptr(loopOutDuration, &freeCb, data);
+        jerry_value_free(loopOutDuration);
+
     }
     //smooth(width=.2, samples=5, t=time)
 
-    auto loopIn = jerry_function_external(_loopIn);
-    jerry_object_set_sz(context, "loopIn", loopIn);
-    jerry_object_set_native_ptr(loopIn, nullptr, exp);
-    jerry_value_free(loopIn);
-
-    auto loopOut = jerry_function_external(_loopOut);
-    jerry_object_set_sz(context, "loopOut", loopOut);
-    jerry_object_set_native_ptr(loopOut, nullptr, exp);
-    jerry_value_free(loopOut);
-
-    auto loopInDuration = jerry_function_external(_loopInDuration);
-    jerry_object_set_sz(context, "loopInDuration", loopInDuration);
-    jerry_object_set_native_ptr(loopInDuration, nullptr, exp);
-    jerry_value_free(loopInDuration);
-
-    auto loopOutDuration = jerry_function_external(_loopOutDuration);
-    jerry_object_set_sz(context, "loopOutDuration", loopOutDuration);
-    jerry_object_set_native_ptr(loopOutDuration, nullptr, exp);
-    jerry_value_free(loopOutDuration);
-
     auto key = jerry_function_external(_key);
     jerry_object_set_sz(context, "key", key);
     jerry_object_set_native_ptr(key, nullptr, exp);
