@@ -38,13 +38,15 @@ public:
     float frameNo = 0.0f;               //current frame number
     float frameCnt = 0.0f;
     float frameDuration = 0.0f;
+    float frameRate = 0.0f;
 
     LottieBuilder* builder;
     LottieComposition* comp = nullptr;
 
+    Key key;
     char* dirName = nullptr;            //base resource directory
     bool copy = false;                  //"content" is owned by this loader
-    bool overriden = false;             //overridden properties with slots
+    bool overridden = false;             //overridden properties with slots
     bool rebuild = false;               //require building the lottie scene
 
     LottieLoader();
@@ -55,7 +57,7 @@ public:
     bool resize(Paint* paint, float w, float h) override;
     bool read() override;
     Paint* paint() override;
-    bool override(const char* slot);
+    bool override(const char* slot, bool byDefault = false);
 
     //Frame Controls
     bool frame(float no) override;
@@ -70,10 +72,12 @@ public:
     bool segment(const char* marker, float& begin, float& end);
 
 private:
+    bool ready();
     bool header();
     void clear();
     float startFrame();
     void run(unsigned tid) override;
+    void release();
 };
 
 
