@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2023 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2023 - 2025 the ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,11 +43,12 @@ struct TtfLoader : public FontLoader
     ~TtfLoader();
 
     using FontLoader::open;
+    using FontLoader::read;
+
     bool open(const string& path) override;
     bool open(const char *data, uint32_t size, bool copy) override;
-    bool transform(Paint* paint, float fontSize, bool italic) override;
-    bool request(Shape* shape, char* text) override;
-    bool read() override;
+    float transform(Paint* paint, FontMetrics& metrices, float fontSize, bool italic) override;
+    bool read(Shape* shape, char* text, FontMetrics& out) override;
     void clear();
 };
 
