@@ -442,7 +442,7 @@ void main() {                                               \n
 const char* MASK_LUMA_FRAG_SHADER = TVG_COMPOSE_SHADER(
 uniform sampler2D uSrcTexture;                                                                          \n
 uniform sampler2D uMaskTexture;                                                                         \n
-                                                                                                        \n
+                                                                                                     \n
 in vec2 vUV;                                                                                            \n
                                                                                                         \n
 out vec4 FragColor;                                                                                     \n
@@ -455,8 +455,7 @@ void main() {
         maskColor = vec4(maskColor.rgb / maskColor.a, maskColor.a);                                     \n
     }                                                                                                   \n
                                                                                                         \n
-    FragColor =                                                                                         \n
-        srcColor * (0.299 * maskColor.r + 0.587 * maskColor.g + 0.114 * maskColor.b) * maskColor.a;     \n
+    FragColor = srcColor * dot(maskColor.rgb, vec3(0.2125, 0.7154, 0.0721)) * maskColor.a;              \n
 }                                                                                                       \n
 );
 
@@ -472,7 +471,7 @@ void main() {
     vec4 srcColor = texture(uSrcTexture, vUV);                                      \n
     vec4 maskColor = texture(uMaskTexture, vUV);                                    \n
                                                                                     \n
-    float luma = (0.299 * maskColor.r + 0.587 * maskColor.g + 0.114 * maskColor.b); \n
+    float luma = dot(maskColor.rgb, vec3(0.2125, 0.7154, 0.0721));                  \n
     FragColor = srcColor * (1.0 - luma);                                            \n
 }                                                                                   \n
 );
@@ -691,9 +690,9 @@ const char* COLOR_DODGE_BLEND_FRAG = COMPLEX_BLEND_HEADER R"(
         vec4 dstColor = texture(uDstTexture, vUV);
 
         FragColor = vec4(
-            1.0 - srcColor.r > 0.0 ? dstColor.r / (1.0 - srcColor.r) : dstColor.r,
-            1.0 - srcColor.g > 0.0 ? dstColor.g / (1.0 - srcColor.g) : dstColor.g,
-            1.0 - srcColor.b > 0.0 ? dstColor.b / (1.0 - srcColor.b) : dstColor.b,
+            srcColor.r < 1.0 ? dstColor.r / (1.0 - srcColor.r) : (dstColor.r > 0.0 ? 1.0 : 0.0),
+            srcColor.g < 1.0 ? dstColor.g / (1.0 - srcColor.g) : (dstColor.g > 0.0 ? 1.0 : 0.0),
+            srcColor.b < 1.0 ? dstColor.b / (1.0 - srcColor.b) : (dstColor.b > 0.0 ? 1.0 : 0.0),
             1.0
         );
     }
@@ -705,9 +704,9 @@ const char* COLOR_BURN_BLEND_FRAG = COMPLEX_BLEND_HEADER R"(
         vec4 dstColor = texture(uDstTexture, vUV);
 
         FragColor = vec4(
-            srcColor.r > 0.0 ? (1.0 - (1.0 - dstColor.r) / srcColor.r) : dstColor.r,
-            srcColor.g > 0.0 ? (1.0 - (1.0 - dstColor.g) / srcColor.g) : dstColor.g,
-            srcColor.b > 0.0 ? (1.0 - (1.0 - dstColor.b) / srcColor.b) : dstColor.b,
+            srcColor.r > 0.0 ? (1.0 - (1.0 - dstColor.r) / srcColor.r) : (dstColor.r < 1.0 ? 0.0 : 1.0),
+            srcColor.g > 0.0 ? (1.0 - (1.0 - dstColor.g) / srcColor.g) : (dstColor.g < 1.0 ? 0.0 : 1.0),
+            srcColor.b > 0.0 ? (1.0 - (1.0 - dstColor.b) / srcColor.b) : (dstColor.b < 1.0 ? 0.0 : 1.0),
             1.0
         );
     }
