@@ -20,6 +20,9 @@
  * SOFTWARE.
  */
 
+#ifndef _TVG_WG_RENDER_DATA_H_
+#define _TVG_WG_RENDER_DATA_H_
+
 #include "tvgWgPipelines.h"
 #include "tvgWgGeometry.h"
 
@@ -30,10 +33,13 @@ struct WgMeshData {
     size_t vertexCount{};
     size_t indexCount{};
 
-    void draw(WGPURenderPassEncoder renderPassEncoder);
-    void drawImage(WGPURenderPassEncoder renderPassEncoder);
+    void draw(WgContext& context, WGPURenderPassEncoder renderPassEncoder);
+    void drawFan(WgContext& context, WGPURenderPassEncoder renderPassEncoder);
+    void drawImage(WgContext& context, WGPURenderPassEncoder renderPassEncoder);
 
-    void update(WgContext& context, WgGeometryData* geometryData);
+    void update(WgContext& context, const WgVertexBuffer& vertexBuffer);
+    void update(WgContext& context, const WgVertexBufferInd& vertexBufferInd);
+    void update(WgContext& context, const Point pmin, const Point pmax);
     void release(WgContext& context);
 };
 
@@ -48,11 +54,13 @@ public:
 };
 
 struct WgMeshDataGroup {
-    static WgMeshDataPool* MeshDataPool;
+    static WgMeshDataPool* gMeshDataPool;
 
     Array<WgMeshData*> meshes{};
-
-    void update(WgContext& context, WgGeometryDataGroup* geometryDataGroup);
+    
+    void append(WgContext& context, const WgVertexBuffer& vertexBuffer);
+    void append(WgContext& context, const WgVertexBufferInd& vertexBufferInd);
+    void append(WgContext& context, const Point pmin, const Point pmax);
     void release(WgContext& context);
 };
 
@@ -60,18 +68,25 @@ struct WgImageData {
     WGPUTexture texture{};
     WGPUTextureView textureView{};
 
-    void update(WgContext& context, Surface* surface);
+    void update(WgContext& context, RenderSurface* surface);
     void release(WgContext& context);
 };
 
 enum class WgRenderSettingsType { None = 0, Solid = 1, Linear = 2, Radial = 3 };
+enum class WgRenderRasterType { Solid = 0, Gradient, Image };
 
 struct WgRenderSettings
 {
-    WgBindGroupSolidColor bindGroupSolid{};
-    WgBindGroupLinearGradient bindGroupLinear{};
-    WgBindGroupRadialGradient bindGroupRadial{};
+    WGPUBuffer bufferGroupSolid{};
+    WGPUBindGroup bindGroupSolid{};
+    WGPUTexture texGradient{};
+    WGPUTextureView texViewGradient{};
+    WGPUBuffer bufferGroupGradient{};
+    WGPUBuffer bufferGroupTransfromGrad{};
+    WGPUBindGroup bindGroupGradient{};
     WgRenderSettingsType fillType{};
+    WgRenderRasterType rasterType{};
+    bool skip{};
 
     void update(WgContext& context, const Fill* fill, const uint8_t* color, const RenderUpdateFlag flags);
     void release(WgContext& context);
@@ -79,11 +94,20 @@ struct WgRenderSettings
 
 struct WgRenderDataPaint
 {
-    WgBindGroupPaint bindGroupPaint{};
+    WGPUBuffer bufferModelMat{};
+    WGPUBuffer bufferBlendSettings{};
+    WGPUBindGroup bindGroupPaint{};
+    RenderRegion viewport{};
+    RenderRegion aabb{};
+    float opacity{};
+    Array<WgRenderDataPaint*> clips;
 
     virtual ~WgRenderDataPaint() {};
     virtual void release(WgContext& context);
-    virtual uint32_t identifier() { return TVG_CLASS_ID_UNDEFINED; };
+    virtual Type type() { return Type::Undefined; };
+
+    void update(WgContext& context, const tvg::Matrix& transform, tvg::ColorSpace cs, uint8_t opacity);
+    void updateClips(tvg::Array<tvg::RenderData> &clips);
 };
 
 struct WgRenderDataShape: public WgRenderDataPaint
@@ -91,15 +115,24 @@ struct WgRenderDataShape: public WgRenderDataPaint
     WgRenderSettings renderSettingsShape{};
     WgRenderSettings renderSettingsStroke{};
     WgMeshDataGroup meshGroupShapes{};
+    WgMeshDataGroup meshGroupShapesBBox{};
+    WgMeshData meshDataBBox{};
     WgMeshDataGroup meshGroupStrokes{};
-    WgMeshData meshBBoxShapes{};
-    WgMeshData meshBBoxStrokes{};
+    WgMeshDataGroup meshGroupStrokesBBox{};
+    Point pMin{};
+    Point pMax{};
     bool strokeFirst{};
+    FillRule fillRule{};
 
-    void updateMeshes(WgContext& context, const RenderShape& rshape);
+    void appendShape(WgContext context, const WgVertexBuffer& vertexBuffer);
+    void appendStroke(WgContext context, const WgVertexBufferInd& vertexBufferInd);
+    void updateBBox(Point pmin, Point pmax);
+    void updateAABB(const Matrix& tr);
+    void updateMeshes(WgContext& context, const RenderShape& rshape, const Matrix& tr);
+    void proceedStrokes(WgContext context, const RenderStroke* rstroke, float tbeg, float tend, const WgVertexBuffer& buff);
     void releaseMeshes(WgContext& context);
     void release(WgContext& context) override;
-    uint32_t identifier() override { return TVG_CLASS_ID_SHAPE; };
+    Type type() override { return Type::Shape; };
 };
 
 class WgRenderDataShapePool {
@@ -114,11 +147,12 @@ public:
 
 struct WgRenderDataPicture: public WgRenderDataPaint
 {
-    WgBindGroupPicture bindGroupPicture{};
+    WGPUBindGroup bindGroupPicture{};
     WgImageData imageData{};
     WgMeshData meshData{};
 
-    void update(WgContext& context);
     void release(WgContext& context) override;
-    uint32_t identifier() override { return TVG_CLASS_ID_PICTURE; };
+    Type type() override { return Type::Picture; };
 };
+
+#endif // _TVG_WG_RENDER_DATA_H_
